#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Usage: perl test.pl
#
# Check some things in ../recurrence-guess.gp

use 5.010;
use warnings;
use strict;
use File::Slurp;
use FindBin;
use File::HomeDir;
use HTML::Entities;
use IPC::Run;

# uncomment this to run the ### lines
# use Smart::Comments;

my $filename = File::Spec->rel2abs
  (File::Spec->catfile($FindBin::Bin,
                       File::Spec->updir,
                       'recurrence-guess.gp'));
my $str = File::Slurp::read_file($filename);
my $bad = 0;

{
  my $count = 0;
  while ($str =~ /^.*DEBUG.*/mg) {
    my $line = $&;
    my $linenum = pos_linenum($str);
    $count++;
    unless ($line =~ /^\s*\\\\/) {
      print "$filename:$linenum: uncommented DEBUG print\n";
      $bad++;
    }
  }
  print "$count DEBUG lines\n";
}

{
  my $input = <<"HERE";
read("$filename");
recurrence_guess([1, 3, 9, 25, 65, 161, 385, 897, 2049, 4609]);
HERE
  IPC::Run::run(['gp','-q','-f'],
                '<',\$input,
                '>',\my $guess_str);
  # print $guess_str;

  # ~/tux/web/pari-recurrence-guess/index.html
  my $html_filename = File::Spec->catfile(File::HomeDir->my_home,
                                          'tux','web','pari-recurrence-guess',
                                          'index.html');
  my $html_str = File::Slurp::read_file($html_filename);
  $html_str =~ m{<blockquote>\n<pre>(.*?)</pre>}s or die "HTML pre not found";
  my $html_guess = $1 . "\n\n";
  my $html_linenum = pos_linenum($html_str,$-[1]);

  $guess_str = <<'HERE' . $guess_str;
read("recurrence-guess.gp");
recurrence_guess([1, 3, 9, 25, 65, 161, 385, 897, 2049, 4609]);
=>

HERE
  $guess_str = HTML::Entities::encode_entities($guess_str,'<>&');

  unless ($guess_str eq $html_guess) {
    print "$html_filename:$html_linenum: recurrence_guess output different\n";
    File::Slurp::write_file('/tmp/recurrence-guess-output.txt',$guess_str);
    File::Slurp::write_file('/tmp/recurrence-guess-html.txt',  $html_guess);
    IPC::Run::run(['diff','-u',
                   '/tmp/recurrence-guess-html.txt',
                   '/tmp/recurrence-guess-output.txt']);
    # print "-------------html:\n";
    # print $html_guess;
    # print "-------------want:\n";
    # print $guess_str;
  }
}

exit ($bad ? 1 : 0);

#------------------------------------------------------------------------------
sub pos_linenum {
  my ($str, $pos) = @_;
  $pos //= pos($_[0]);
  $str = substr($str, 0, $pos);
  return 1 + scalar($str =~ tr/\n//);
}

#------------------------------------------------------------------------------

exit ($bad ? 1 : 0);
